#!/bin/sh

# This sctipt does the actual work of update - after we reboot.

# assumptions :-
#   - kernel file name is kernel.img
#   - rootfs filename is rootfs.jffs2

set -x
need_update=1
update_nand_timing()
{
        ${MEMTOOL} write 0xd1800040 0x884a
        ${MEMTOOL} write 0xd1800048 0x10101010
        ${MEMTOOL} write 0xd180004c 0x10101010
}
find_mtddev()
{
	local part=${1}
	dev=$(cat /proc/mtd | grep \"${part}\" | cut -d: -f1)
	echo "/dev/$dev"
}

verify_checksum()
{
	local image_name=${1}
	md5=$(md5sum ${update_base}/${image_name} | cut -d" " -f1)
#	echo $md5
	md5_read=$(cat ${update_base}/checksum | grep ${image_name} | cut -d" " -f1)
#	echo $md5_read
	if [ "$md5" == "$md5_read" ]; then
		echo "cool ${image_name} file is OK"
#		need_update=1
	else
		echo "${image_name} file is corrupted.."
#		need_update=0
#		rm -rf ${update_base}/*
#		exit 0 
	fi
		
}

do_update()
{
	local mtdpart=${1}
	local image_name=${2}
	local erase_opt=${3}
	local nand_opt=${4}
	local start_offset=${5:-0}
	local block_count=${6:-0}

	devname=`find_mtddev ${mtdpart}`
	update_nand_timing
	[ -e ${devname} ] && ${FLASH_ERASE} ${erase_opt} ${devname} ${start_offset} ${block_count}
	[ -e ${devname} ] && ${NANDWRITE} -s ${start_offset} ${nand_opt} ${devname} ${image_name}

	return 0

}


do_backup_config()
{
	cmd=${1}
	mtdpart=${2}
	backupfile=${3}
	srcpath=${4}

	[ ! -d /jffs ] && mkdir /jffs
	grep mtd:ubifs /proc/mounts
	if [ $? -ne 0 ]; then
		mount -t jffs2 mtd:ubifs /jffs
	fi
	if [ "$cmd" == "save" ]; then
		tar -C /jffs -cf ${backupfile} ${srcpath}/uxp ${srcpath}/uxp_backup ${srcpath}/mode
	elif [ "$cmd" == "restore" ]; then
		tar -C /jffs -xf ${backupfile} 
		# Special cludge for pre 1.5 versions.
		# remove the linked files - so that they are taken care off.
		if [ -L /jffs/etc/config/uxp -o -L /jffs/etc/config/uxp_backup  ]; then
			rm -f /jffs/etc/config/uxp
			rm -f /jffs/etc/config/uxp_backup
			[ ! -d /tmp/readable ] && mkdir /tmp/readable
			mount -t jffs2 mtd:mfgdata /tmp/readable
			[ -f /tmp/readable/config/uxp ] && cp /tmp/readable/config/uxp /jffs/etc/config/
			[ -f /tmp/readable/config/uxp_backup ] && cp /tmp/readable/config/uxp_backup /jffs/etc/config/
			umount /tmp/readable
		fi
	fi

	umount /jffs
	sync

}

check_block_first_ff()
{
	local devname check_str
	local mtdpart=${1}
	local start=${2}
	devname=`find_mtddev ${mtdpart}`
	if [ -n "${devname}" -a -e "${devname}" ]; then
		dd if=${devname} of=/tmp/lic_dump bs=4k count=1
		# ${NANDDUMP} --omitoob --bb=skipbad -s ${start} -f /tmp/lic_dump -l 0x800 ${devname}
		check_str=$(hexdump -e '"""%04X"' -v -n 16 /tmp/lic_dump | sed -e 's/FF*//g')
		if [ x${check_str} = x ] ; then
			return 0
		else
			return 1
		fi
	fi
	return 2
}


update_base=`pwd`/update.d

MTD_BIN="/usr/sbin"
FMT_LIC_START_OFFSET="0x40000"

# Set up the required environment needed for update
[ -d /sys ] || mkdir /sys
[ -d /dev ] || mkdir /dev
[ -d /proc ] || mkdir /proc
[ -d /tmp ] || mkdir /tmp

mount -t proc proc /proc
mount -t sysfs sysfs /sys
mount -t tmpfs tmpfs /tmp
mount -t tmpfs dev /dev

[ -d /dev/shm ] || mkdir /dev/shm
mount -t tmpfs shm /dev/shm
[ -d /dev/pts ] || mkdir /dev/pts
mount -t devpts none /dev/pts

echo "/sbin/mdev" > /sys/kernel/uevent_helper
mdev -s

# Now run the mtd-utils

mkdir /scratch
cd /scratch
tar -zxf ${update_base}/update-utils.tar.gz 


FLASH_ERASE=.${MTD_BIN}/flash_erase
NANDWRITE=.${MTD_BIN}/nandwrite
MEMTOOL=./usr/local/bin/memtool

need_reboot=0

if [ -f ${update_base}/xloader.img ]; then
	verify_checksum xloader.img
fi

if [ -f ${update_base}/u-boot.img ]; then
	verify_checksum u-boot.img
fi
if [ -f ${update_base}/kernel.img ]; then
	verify_checksum kernel.img
fi
if [ -f ${update_base}/rootfs.jffs2 ]; then
	verify_checksum rootfs.jffs2
fi
if [ -f ${update_base}/mfg.jffs2 ]; then
	verify_checksum mfg.jffs2
fi
if [ -f ${update_base}/boardinfo.bin ]; then
	verify_checksum boardinfo.bin
fi

if [ $need_update -eq 1 ]; then
	if [ -f ${update_base}/xloader.img ]; then
       		do_update xloader ${update_base}/xloader.img "" --pad
       		need_reboot=1
	fi

	if [ -f ${update_base}/u-boot.img ]; then
        	do_update u-boot ${update_base}/u-boot.img "" --pad
        	need_reboot=1
	fi


# Lets check FomatLic.bin and see if it all empty
	if check_block_first_ff secret ${FMT_LIC_START_OFFSET}; then
		do_update secret ${update_base}/FormatLic.bin "" "--pad" ${FMT_LIC_START_OFFSET} 1 
	fi

	do_backup_config save ubifs /scratch/config.tar etc/config
	do_update ubifs ${update_base}/rootfs.jffs2 "" ""
	do_backup_config restore ubifs /scratch/config.tar etc/config
	do_update kernel ${update_base}/kernel.img "" --pad

	if [ -f ${update_base}/mfg.jffs2 ]; then
        	do_update mfgdata ${update_base}/mfg.jffs2 "" ""
	fi

	if [ -f ${update_base}/boardinfo.bin ]; then
	# This area need to be updated with some more checks in future release.
		if check_block_first_ff boardinfo 0x0; then
			echo "Update boardinfo"
        		do_update boardinfo ${update_base}/boardinfo.bin "" --pad
		else
			echo "No need to update boardinfo"
		fi
	fi
else
	return 0
fi
# tar -xf /update/update.tar -O ./rootfs.jffs2 | /tmp/usr/sbin/nandwrite --pad /dev/mtd4 -


